#!/usr/bin/env python
# Copyright (c) 2023 VMware Carbon Black, all rights reserved
################################################################################

import argparse
import logging
import re
import subprocess
import sys
from typing import Optional, Tuple

import requests

# noinspection PyUnresolvedReferences
requests.packages.urllib3.disable_warnings()

logger = logging.getLogger(__name__)


################################################################################
# Setup functions
################################################################################


class BlankLinesHelpFormatter(argparse.HelpFormatter):
    """
    Add empty line if help ends with \n
    """

    def _split_lines(self, text: str, width: int):
        # noinspection PyProtectedMember
        lines = super()._split_lines(text, width)
        if text.endswith('\n'):
            lines += ['']
        return lines


def build_cli_parser() -> argparse.Namespace:
    """
    Build the set of accepted options.
    """
    # noinspection PyTypeChecker
    parser = argparse.ArgumentParser(description="Get hello service weighting for production environments.",
                                     formatter_class=BlankLinesHelpFormatter)

    # for each supported output type, add an option
    parser.add_argument("--debug", action="store_true", dest="debug", default=False,
                        help="enable debug logging\n")
    parser.add_argument("--only", action="store", dest="only", default=None,
                        help="select a specific environment")

    args = parser.parse_args()
    return args


def check_aws() -> Optional[Exception]:
    """
    Verify that we have access to AWS, return exception if we do not.
    :return: None if ok, exception with message otherwise
    """
    # noinspection SpellCheckingInspection
    cmd = ["aws", "elbv2", "describe-rules", "--region", "us-east-1", "--rule-arn",
           "arn:aws:elasticloadbalancing:us-east-1:132308400445:listener-rule/app/prod05-devices-alb/61007b271b249f31/f60f1977abb1915c/df949273bf226385"]
    try:
        logger.debug(f"TRY: '{' '.join(cmd)}'")
        check = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        stderr = check.stderr.decode('utf-8')
        if check.returncode != 0:
            return RuntimeError(f"{stderr}")
    except Exception as err:
        return err

    # Everything ok
    return None


def get_weighting(env: str) -> Tuple[str, Optional[Exception], bool]:
    """
    Get the weighting information for the given environment.
    :param env: environment to look for in the data
    :return: information about 443 and 5443, or any exception
    """

    def _make_call(cmd1) -> tuple[str, Optional[Exception]]:
        try:
            logger.debug(f"Using: {cmd}")
            check = subprocess.run(cmd1, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            if check.returncode != 0:
                stderr = check.stderr.decode('utf-8')
                return "", RuntimeError(f"{stderr}")
            else:
                stdout = check.stdout.decode('utf-8')
                return stdout, None
        except Exception as err1:
            return "", err1

    def _clean(weight: str) -> int:
        p = re.compile(r"\[\s+(\d+)\s+]")
        m = re.match(p, weight)
        if m is not None:
            return int(m.group(1))
        else:
            logger.warning(f"Unable to process weighting of '{weight}'")
            return -999

    query = "Rules[0].Actions[0].ForwardConfig.TargetGroups[?(contains(@.TargetGroupArn, 'mcs2'))].Weight"
    region = DATA[env]["region"].strip()
    arn1 = DATA[env]["443"].strip()
    arn2 = DATA[env]["5443"].strip()
    notes = DATA[env].get("notes", None)

    # noinspection SpellCheckingInspection
    cmd = ["aws", "elbv2", "describe-rules", "--rule-arn", arn1, "--region", region, "--query", query]
    weight1, err = _make_call(cmd)
    if err is not None:
        return "", err, False
    weight1 = _clean(weight1)

    # noinspection SpellCheckingInspection
    cmd = ["aws", "elbv2", "describe-rules", "--rule-arn", arn2, "--region", region, "--query", query]
    weight2, err = _make_call(cmd)
    if err is not None:
        return "", err, False
    weight2 = _clean(weight2)

    answer = f"port 443: {weight1:-3d}%, port 5443: {weight2:-3d}%"
    if notes is not None:
        answer += f"     -- ({notes})"

    # check for different 433 and 5443 values
    diffs = weight1 != weight2

    return answer, None, diffs


################################################################################
# Begin Main
################################################################################

# noinspection SpellCheckingInspection
DATA = {
    "EAP01": {
        "region": "us-east-1",
        "443": "arn:aws:elasticloadbalancing:us-east-1:132308400445:listener-rule/app/eap01-devices-alb/b0a66b36b738710e/8bb3946b300fcbb4/ab20170d28b39cc4",
        "5443": "arn:aws:elasticloadbalancing:us-east-1:132308400445:listener-rule/app/eap01-devices-alb/b0a66b36b738710e/eee7b510db5702b7/7b941e1901591abe"
    },
    "PROD01": {
        "region": "us-east-1",
        "443": "arn:aws:elasticloadbalancing:us-east-1:132308400445:listener-rule/app/prod01-devices-alb/f0a434b76a197104/f2892a4fb9f390eb/7c48456b01457cf4",
        "5443": "arn:aws:elasticloadbalancing:us-east-1:132308400445:listener-rule/app/prod01-devices-alb/f0a434b76a197104/a6aa5b32c4e45245/61a432b54a13d0d2"
    },
    "PROD02": {
        "region": "us-east-1",
        "443": "arn:aws:elasticloadbalancing:us-east-1:132308400445:listener-rule/app/prod02-devices-alb/14b5069e226cee31/63b398cc1bd82cb2/e11f3cf4041f8403",
        "5443": "arn:aws:elasticloadbalancing:us-east-1:132308400445:listener-rule/app/prod02-devices-alb/14b5069e226cee31/1e53a966d9b8b94d/5b22df3259592162"
    },
    "PROD05": {
        "region": "us-east-1",
        "443": "arn:aws:elasticloadbalancing:us-east-1:132308400445:listener-rule/app/prod05-devices-alb/61007b271b249f31/f60f1977abb1915c/df949273bf226385",
        "5443": "arn:aws:elasticloadbalancing:us-east-1:132308400445:listener-rule/app/prod05-devices-alb/61007b271b249f31/e01eb2e8fe6d6a52/7eb955acc7cc6e0b"
    },
    "PROD06": {
        "region": "eu-central-1",
        "notes": "Frankfurt",
        "443": "arn:aws:elasticloadbalancing:eu-central-1:132308400445:listener-rule/app/prod06-devices-alb/93b64003740b35bc/1018a06e0758f1c2/4dc40d7f8c7913eb",
        "5443": "arn:aws:elasticloadbalancing:eu-central-1:132308400445:listener-rule/app/prod06-devices-alb/93b64003740b35bc/027135deb14702e9/de1a507a795802a7"
    },
    "PRODNRT": {
        "region": "ap-northeast-1",
        "notes": "APNE1: Tokyo",
        "443": "arn:aws:elasticloadbalancing:ap-northeast-1:132308400445:listener-rule/app/prodnrt-devices-alb/ed26afb82e3447ab/72e62f662950c3b3/2708d1dfbb8f38c8",
        "5443": "arn:aws:elasticloadbalancing:ap-northeast-1:132308400445:listener-rule/app/prodnrt-devices-alb/ed26afb82e3447ab/f6214123270f48e0/e647d6f5e4cb2585"
    },
    "PRODSYD": {
        "region": "ap-southeast-2",
        "notes": "APSE2: Sydney",
        "443": "arn:aws:elasticloadbalancing:ap-southeast-2:132308400445:listener-rule/app/prodsyd-devices-alb/da633c867dd93253/5d5f5e663ddb54b3/bd1281d004af02c3",
        "5443": "arn:aws:elasticloadbalancing:ap-southeast-2:132308400445:listener-rule/app/prodsyd-devices-alb/da633c867dd93253/7da93b639e8c7fc7/cd96fae8c63937e9"
    },
}


def main():
    try:
        args = build_cli_parser()
    except Exception as err:
        print(f"***** Failed while building command arguments: {err}")
        sys.exit(1)
    level = logging.DEBUG if args.debug else logging.INFO
    logging.getLogger().setLevel(level)
    if args.only is not None:
        if args.only not in DATA:
            print(f"***** For '--only', you must specify one of: {sorted(DATA.keys())}")
            sys.exit(1)

    fmt = '%(asctime)s - [%(process)d] %(levelname)7s [%(name)s:%(lineno)d] %(message)s'
    logging.basicConfig(level=level, format=fmt)
    check = check_aws()
    if check is not None:
        print(f"***** Unable to contact AWS: {check}\n ... do you need to run 'gimme-aws-creds'?")
        sys.exit(1)

    saw_diffs = False
    print("Current mosaic2 weights for /services/hello")
    print("=" * 80)
    use_list = sorted(DATA.keys()) if args.only is None else [args.only]
    for env in use_list:
        report = f"  {env:10s} -- "
        info, ex, diffs = get_weighting(env)
        if ex is not None:
            print(f"***** Failure on data gather: {ex}")
            continue
        report += info
        if diffs:
            report = "!" + report
            saw_diffs = True
        else:
            report = " " + report
        print(report)
    print("=" * 80)
    if saw_diffs:
        print("! indicates that 443 and 5443 weights differ")


if __name__ == '__main__':
    main()
